
/*
*******************************************************************************
Include Part
*******************************************************************************
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include  <string.h>
#include  <stdio.h>

#include "main.h"
/*
********************************************************************************
Function For UART0
********************************************************************************
*/
SIGNAL(SIG_UART0_RECV)
{
	rx0_buf[p_rx0_wr++] = UDR0;
	if(rx0_buf[p_rx0_wr-1] == LF_VALUE)lf_check_flag++;
	if (p_rx0_wr > UART0_BUF_SIZE-1)p_rx0_wr = 0;	
}

void Init_UART0(unsigned char baudrate)
{
	unsigned int i;
	
	UBRR0H      = 0;
	if(baudrate == BAUD_9600){UBRR0L = 71;}/* 9600 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_19200){UBRR0L = 35;}/* 19200 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_38400){UBRR0L = 17;}/* 38400 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_57600){UBRR0L = 11;}/* 57600 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_115200){UBRR0L = 5;} /* 115200 BAUD at 11.0592MHz*/
	else {UBRR0L = 71;};/* default 9600 BAUD at 11.0592MHz*/
	UCSR0B = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);
	p_rx0_wr = 0;
	p_rx0_rd = 0;
	for (i=0; i<UART0_BUF_SIZE; i++) rx0_buf[i] = 0;
 }

unsigned char PutChar0 (unsigned char c)
{
	while (!(UCSR0A & 0x20)) ;
	UDR0 = c;
	return 0;
}

void DispStr0 (char *s)
{
	while (*s != '\0') {
		PutChar0(*s++);
	}
}

void DispStr0_line (char *s)
{
	while (*s != '\0') {
		PutChar0(*s++);
	}
}

unsigned int Check_Rx_Buf0(void)
{
	unsigned int len;

	if (p_rx0_wr == p_rx0_rd) {
		len = 0;
	}
	else {
		if (p_rx0_wr > p_rx0_rd) len = p_rx0_wr - p_rx0_rd;
		else len = UART0_BUF_SIZE + p_rx0_wr - p_rx0_rd;
	}
	return len;
}

unsigned char GetChar0 (void)
{
	unsigned char ch;
    
	ch = rx0_buf[p_rx0_rd];
	p_rx0_rd++;
	if (p_rx0_rd > UART0_BUF_SIZE-1) p_rx0_rd = 0;
	return ch;
}
/*
********************************************************************************
Function For UART1
********************************************************************************
*/
SIGNAL(SIG_UART1_RECV)
{
	rx1_buf[p_rx1_wr++] = UDR1;
	if (p_rx1_wr > UART1_BUF_SIZE-1)p_rx1_wr = 0;
}

void Init_UART1(unsigned char baudrate)
{
	unsigned int i;

	UBRR1H      = 0;
	if(baudrate == BAUD_9600){UBRR1L = 71;}/* 9600 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_19200){UBRR1L = 35;}/* 19200 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_38400){UBRR1L = 17;}/* 38400 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_57600){UBRR1L = 11;}/* 57600 BAUD at 11.0592MHz*/
	else if(baudrate == BAUD_115200){UBRR1L = 5;} /* 115200 BAUD at 11.0592MHz*/
	else {UBRR1L = 71;};/* default 9600 BAUD at 11.0592MHz*/
	UCSR1B = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);
	p_rx1_wr = 0;
	p_rx1_rd = 0;
	for (i=0; i<UART1_BUF_SIZE; i++) rx1_buf[i] = 0;
}

unsigned char PutChar1 (unsigned char c)
{
	while (!(UCSR1A & 0x20)) ;
	UDR1 = c;
	return 0;
}

void DispStr1 (char *s)
{
	while (*s != '\0') {
		PutChar1(*s++);
	}
	PutChar1(0x0a);
	PutChar1(0x0d);
}

void DispStr1_line (char *s)
{
	while (*s != '\0') {
		PutChar1(*s++);
	}
}

unsigned int Check_Rx_Buf1(void)
{
	unsigned int len;

	if (p_rx1_wr == p_rx1_rd) {
		len = 0;
	}
	else {
		if (p_rx1_wr > p_rx1_rd) len = p_rx1_wr - p_rx1_rd;
		else len = UART1_BUF_SIZE + p_rx1_wr - p_rx1_rd;
	}
	return len;
}

unsigned char GetChar1 (void)
{
	unsigned char ch;
    
	ch = rx1_buf[p_rx1_rd];
	p_rx1_rd++;
	if (p_rx1_rd > UART1_BUF_SIZE-1) p_rx1_rd = 0;
	return ch;
}
/*
********************************************************************************
Function For Timer0 & LED
********************************************************************************
*/
SIGNAL(SIG_OVERFLOW0)
{

	timer0_counter++;
	transfer_interval++;
	if (timer0_counter > 200 )
	{	
		timer0_counter = 0;
	}
	if (transfer_interval > 60000 )
	{	
		transfer_interval = 0;
	}
	status_led();
}

void init_timer0(void)
{

	TIMSK |= 1 << TOIE0;
	TCNT0 = 0;
	TCCR0 = 5;
	timer0_counter = 0;
}

void status_led(void)
{
	if(status_sequence == BEFORE_CONNECT)
	{
		if (timer0_counter < 100 )
		{
			if(read_avr_port == DIP_1)
				write_port_value(WRITE_LED_1_ON);
			else if(read_avr_port == DIP_2)
				write_port_value(WRITE_LED_2_ON);
		}
		else if (timer0_counter < 200  )
		{
			if(read_avr_port == DIP_1)
				write_port_value(WRITE_LED_1_OFF);
			else if(read_avr_port == DIP_2)
				write_port_value(WRITE_LED_2_OFF);
		}
	}
	else
	{
		if(read_avr_port == DIP_1)
			write_port_value(WRITE_LED_1_ON);
		else if(read_avr_port == DIP_2)
			write_port_value(WRITE_LED_2_ON);
	}
}
/*
********************************************************************************
Get Bluetooth 
Address from Device
********************************************************************************
*/
void get_bluetooth_address(void)
{
	unsigned int i;

	step_slave_bt = STEP_1;
	init_uart0_read_data();
	while(1)
	{
		uart0_length = Check_Rx_Buf0();
		if(uart0_length>0)
		{
			if(lf_check_flag == 2)
			{
				uart0_length = Check_Rx_Buf0();
				for(i = 0;i<uart0_length;i++)
					uart0_read_data[i] = GetChar0();
				switch(step_slave_bt)
				{
					case STEP_1:
						lf_check_flag = LF_CHECK_NON;
						if(!memcmp("\r\nBTWIN Slave mode start\r\n",&uart0_read_data[uart0_length-26],26))
						{
							step_slave_bt++;
						}
						else if(!memcmp("\r\nBTWIN Master mode start\r\n",&uart0_read_data[uart0_length-27],27))
						{
							step_slave_bt++;
						}
						else
							execution_error();
						break;
					case STEP_2:
						lf_check_flag = LF_CHECK_NON;
						if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
						{
							step_slave_bt++;
							DispStr0("at+btinfo?8\r");
						}
						else
							execution_error();
						break;
					case STEP_3:
						lf_check_flag = LF_CHECK_NON;
						if(uart0_length == 16)
						{
							step_slave_bt++;
							DispStr1("[FBD755MC_gcc V0.2]");
							DispStr1("[Bluetooth Device Address]");
							for(i = 0;i < uart0_length;i++)
								PutChar1(uart0_read_data[i]);
						}
						else
							execution_error();
						break;
				}	
				init_uart0_read_data();
			}
		}
	}
}
/*
********************************************************************************
CONVERT HEX INTO CHAR
********************************************************************************
*/
unsigned char Hex2Char(unsigned char c)
{
	if ((c == 0)||(c == 9)||(c > 0&&c < 9))
		return '0' + c;
	if (c >= 10 && c <= 15)
		return 'A' + c - 10;

	return c;
}

/*
********************************************************************************
Function For Delay
********************************************************************************
*/
void wait_1ms(unsigned int cnt)
{
	unsigned int i;

	for (i = 0; i < cnt; i++) wait_1us(1000);
}

void wait_1us(unsigned int cnt)
{
	unsigned int i;

	for (i = 0; i < cnt; i++) ;
}
/*
********************************************************************************
Function For Initialize PORT
********************************************************************************
*/
void init_port (void)
{
	DDRA  = 0x00;
	PORTA = 0x0f;

	DDRB  = 0xff;
	PORTB = 0xf0;
	
	DDRC  = 0x04;
	PORTC = 0x00;
	
	DDRD  = 0xff;
	PORTD = 0x00;
	
	DDRE  = 0xff;
	PORTE = 0x00;
	
	DDRF  = 0xff;
	PORTF = 0x00;
}
/*
********************************************************************************
Function For PORT Read
********************************************************************************
*/
unsigned char read_port_value(unsigned char avr_port_num)
{
	unsigned char return_avr_port_value;
	
	switch(avr_port_num)
	{
		case READ_DIP:
			return_avr_port_value = PINA;
			return_avr_port_value = return_avr_port_value & 0xf0;
			break;
		case READ_SWITCH:
			return_avr_port_value = PINA;
			return_avr_port_value = return_avr_port_value & 0x0f;
			break;
		//FB755AX
		case READ_S_STATUS:
			return_avr_port_value = PINC;
			return_avr_port_value = return_avr_port_value & 0x02;
			break;
		//
		default:
			break;
	}
	return return_avr_port_value;
}
/*
********************************************************************************
Function For PORT Write
********************************************************************************
*/
void write_port_value(unsigned char avr_port_value)
{

	switch(avr_port_value)
	{
		case WRITE_LED_1_ON:
			cbi(PORTB,PB4);
			break;
		case WRITE_LED_1_OFF:
			sbi(PORTB,PB4);
			break;
		case WRITE_LED_2_ON:
			cbi(PORTB,PB5);
			break;
		case WRITE_LED_2_OFF:
			sbi(PORTB,PB5);
			break;
		//FB755AX
		case WRITE_S_CONTROL_HIGH:
			sbi(PORTC,PC2);
			break;
		case WRITE_S_CONTROL_LOW:
			cbi(PORTC,PC2);
			break;
		//
		default:
			break;
	}
}
/*
********************************************************************************
* Function For Init uart0_read_data
********************************************************************************
*/
void init_uart0_read_data(void)
{
	unsigned int i;
	
	for(i = 0;i<50;i++)
		uart0_read_data[i] = 0;
}
/*
********************************************************************************
* Function For error
********************************************************************************
*/
void execution_error(void)
{
	unsigned int i;
	
	DispStr1_line("[ERROR ");
	DispStr1_line(" STEP : ");
	if(read_avr_port == DIP_1)
		PutChar1(Hex2Char(step_slave_bt));
	else if(read_avr_port == DIP_2)
		PutChar1(Hex2Char(step_master_bt));
	DispStr1_line(" UART_LEN : ");
	PutChar1(Hex2Char(uart0_length));
	DispStr1_line(" DATA : ");
	for(i = 0;i < uart0_length;i++)
		PutChar1(uart0_read_data[i]);
	if(read_avr_port == DIP_1)
		step_slave_bt = 50;
	else if(read_avr_port == DIP_2)
		step_master_bt = 50;
	
}
/*
********************************************************************************
* Function For compare slave command
********************************************************************************
*/
void compare_slave_command(void)
{
	unsigned int i;
	
	switch(step_slave_bt)
	{
		case STEP_1:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nBTWIN Slave mode start\r\n",&uart0_read_data[uart0_length-26],26))
			{
				step_slave_bt = STEP_3;
				DispStr1("[STEP_1 : START MSG RECEIVED]");
			}
			else if(!memcmp("\r\nBTWIN Master mode start\r\n",&uart0_read_data[uart0_length-27],27))
			{
				step_slave_bt++;
				DispStr1("[STEP_1 : START MSG RECEIVED]");
			}
			else
				execution_error();
			break;
		case STEP_2:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_slave_bt++;
				DispStr1("[STEP_2 : OK MSG RECEIVED]");
				DispStr1("[         CHANGE ROLE]");				
				DispStr0("at+btrole=s\r");
			}
			else
				execution_error();
			break;
		case STEP_3:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_slave_bt++;
				DispStr1("[STEP_3 : OK MSG RECEIVED]");
				DispStr1("[         SET OPMODE 1]");
				wait_1ms(100);
				DispStr0("at+btopmode,1\r");
			}
			else
				execution_error();
			break;
		case STEP_4:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				if(CONNECT_DEV_NUM > 0x37)
				{
					DispStr1("[ERROR, CHECK CONNECT_DEV_NUM]");
					DispStr1("[         MAX CONNECT_DEV_NUM : 7]");
					step_slave_bt = 50;
				}
				else
				{
					step_slave_bt++;
					DispStr1("[STEP_4 : OK MSG RECEIVED]");
					DispStr1_line("[         SET DEV NUM : ");
					PutChar1(CONNECT_DEV_NUM);
					DispStr1("]");
					wait_1ms(100);
					DispStr0("at+btdev=");
					PutChar0(CONNECT_DEV_NUM);
					PutChar0('\r');
				}
			}
			else
				execution_error();
			break;
		case STEP_5:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_slave_bt++;
				DispStr1("[STEP_5 : OK MSG RECEIVED]");
				DispStr1("[         SET BUF SIZE : 10 BYTE]");
				wait_1ms(100);
				DispStr0("at+btbuff=10\r");
			}
			else
				execution_error();
			break;
		case STEP_6:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_slave_bt++;
				DispStr1("[STEP_6 : OK MSG RECEIVED]");
				DispStr1("[         RESET BLUETOOTH]");
				DispStr0("atz\r");
			}
			else
				execution_error();
			break;
		case STEP_7:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nBTWIN Slave mode start\r\n",&uart0_read_data[uart0_length-26],26))
			{
				step_slave_bt++;
				DispStr1("[STEP_7 : START MSG RECEIVED]");
			}
			else
				execution_error();
			break;
		case STEP_8:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_slave_bt++;
				DispStr1("[STEP_8 : OK MSG RECEIVED]");
				DispStr1("[         EXECUTON SCAN]");
				wait_1ms(100);
				DispStr0("at+btscan\r");
			}
			else
				execution_error();
			break;
		case STEP_9:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_slave_bt++;
				DispStr1("[STEP_9 : OK MSG RECEIVED]");
				DispStr1("[         WAIT CONNECT MSG]");
			}
			else
				execution_error();
			break;
		case STEP_10:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nCONNECT",&uart0_read_data[uart0_length-24],9))
			{
				connect_count++;
				if(connect_count == CONNECT_DEV_NUM)
				{
					step_slave_bt++;
					status_sequence = AFTER_CONNECT;
					write_port_value(WRITE_S_CONTROL_LOW);
					DispStr1_line("[STEP_11 : ");
					for(i = 2;i < uart0_length-2;i++)
						PutChar1(uart0_read_data[i]);
					DispStr1("]");
					DispStr1("[         COMMUNICATION START]");
				}
				else
				{
					step_slave_bt = STEP_10;
					DispStr1_line("[STEP_10 : ");
					for(i = 2;i < uart0_length-2;i++)
						PutChar1(uart0_read_data[i]);
					DispStr1("]");
					DispStr1("[         WAIT NEXT CONNECT MSG]");
				}
			}
			else
				execution_error();
			break;
	}	
	init_uart0_read_data();
}
/*
********************************************************************************
* Function For check stream status port
********************************************************************************
*/
void execution_check_s_status_low(void)
{
	loop_count_1 = 0;
	loop_count_2 = 0;
	s_status_value = OK;
	for(;;)
	{
		read_avr_port = read_port_value(READ_S_STATUS);
		if(read_avr_port == S_STATUS_LOW)break;
		loop_count_1++;
		if(loop_count_1 > 1000)
		{
			loop_count_1 = 0;
			loop_count_2++;
		}
		if(loop_count_2 > 200)
		{
			loop_count_1 = 0;
			loop_count_2 = 0;
			s_status_value = ERROR;
			break;
		}
	}
}

void execution_check_s_status_high(void)
{
	loop_count_1 = 0;
	loop_count_2 = 0;
	s_status_value = OK;
	for(;;)
	{
		read_avr_port = read_port_value(READ_S_STATUS);
		if(read_avr_port == S_STATUS_HIGH)break;
		loop_count_1++;
		if(loop_count_1 > 1000)
		{
			loop_count_1 = 0;
			loop_count_2++;
		}
		if(loop_count_2 > 200)
		{
			loop_count_1 = 0;
			loop_count_2 = 0;
			s_status_value = ERROR;
			break;
		}
	}
}
/*
********************************************************************************
* Function For stream control port & ATOn command & transfer slave data
********************************************************************************
*/
void execution_stream_control(void)
{
	read_avr_port = read_port_value(READ_SWITCH);
	if(read_avr_port == SWITCH_1||read_avr_port == SWITCH_2||
		read_avr_port == SWITCH_3||read_avr_port == SWITCH_4)
	{
		old_switch_value = read_avr_port;
		write_port_value(WRITE_S_CONTROL_HIGH);
		execution_check_s_status_high();
		if(s_status_value == OK)
		{
			switch(old_switch_value)
			{
				case SWITCH_1:
					DispStr0("ato1\r");
					DispStr1("[CHANGE STREAM CONNECT 1]");
					break;
				case SWITCH_2:
					DispStr0("ato2\r");
					DispStr1("[CHANGE STREAM CONNECT 2]");
					break;
				case SWITCH_3:
					DispStr0("ato3\r");
					DispStr1("[CHANGE STREAM CONNECT 3]");
					break;
				case SWITCH_4:
					DispStr0("ato4\r");
					DispStr1("[CHANGE STREAM CONNECT 4]");
					break;
			}
			execution_check_s_status_low();
			if(s_status_value == ERROR)
			{
				write_port_value(WRITE_S_CONTROL_LOW);
				stream_status = ERROR;
				DispStr1("[STREAM STATUS ERROR]");
			}
			else
			{
				write_port_value(WRITE_S_CONTROL_LOW);
				stream_status = OK;
				DispStr1("[STREAM STATUS OK]");
				DispStr1("SENDING DATA : 0ABCDEFGH0");
				DispStr0_line("0ABCDEFGH0");
			}
		}
		else
		{
			write_port_value(WRITE_S_CONTROL_LOW);
			stream_status = ERROR;
			DispStr1("[STREAM STATUS ERROR]");
		}
	}
}
/*
********************************************************************************
* Function For distribute data
********************************************************************************
*/
void execution_get_received_message(void)
{
	unsigned int i;

	if(message_value == STATUS_CON)
	{
		wait_1ms(150);
		for(i = 1;i<10;i++)uart0_read_data[i] = GetChar0();
	}
	else if(message_value == DATA_OK)
	{
		for(i = 1;i<10;i++)uart0_read_data[i] = GetChar0();
	}
}

void execution_put_received_message(void)
{
	unsigned int i;

	if(message_value == DATA_OK)
		for(i = 0;i < 10;i++)PutChar1(uart0_read_data[i]);
	else if(message_value == STATUS_CON)
		for(i = 2;i < 22;i++)PutChar1(uart0_read_data[i]);
	else if(message_value == STATUS_DISCON)
		for(i = 2;i < 25;i++)PutChar1(uart0_read_data[i]);
	DispStr1("");
}

void execution_distribute_received_message(void)
{
	unsigned int i;
	
	message_value = DATA_OK;
	init_uart0_read_data();
	uart0_length = 0;
	uart0_length = Check_Rx_Buf0();
	if(uart0_length>9)
	{
		uart0_read_data[0] = GetChar0();
		switch(uart0_read_data[0])
		{
			case 0x0d:
				message_value = STATUS_CON;
				execution_get_received_message();
				if(uart0_read_data[2] == 'C')
				{
					for(i = 10;i<24;i++)uart0_read_data[i] = GetChar0();
					if(uart0_read_data[23] == 0x0a)
						DispStr1_line("STATUS MESSAGE : ");
					else
						DispStr1_line("CON ERROR : ");
				}
				else if(uart0_read_data[2] == 'D')
				{
					message_value = STATUS_DISCON;
					for(i = 10;i<27;i++)uart0_read_data[i] = GetChar0();
					if(uart0_read_data[26] == 0x0a)
						DispStr1_line("STATUS MESSAGE : ");
					else
						DispStr1_line("DISCON ERROR : ");
				}
				break;
			case 0x30:
				execution_get_received_message();
				if(uart0_read_data[9] == 0x30)
					DispStr1_line("SLAVE 0 DATA : ");
				else
					DispStr1_line("SLAVE 0 ERROR : ");
				break;
			case 0x31:
				execution_get_received_message();
				if(uart0_read_data[9] == 0x31)
					DispStr1_line("MASTER 1 DATA : ");
				else
					DispStr1_line("MASTER 1 ERROR : ");
				break;
			case 0x32:
				execution_get_received_message();
				if(uart0_read_data[9] == 0x32)
					DispStr1_line("MASTER 2 DATA : ");
				else
					DispStr1_line("MASTER 2 ERROR : ");
				break;
			case 0x33:
				execution_get_received_message();
				if(uart0_read_data[9] == 0x33)
					DispStr1_line("MASTER 3 DATA : ");
				else
					DispStr1_line("MASTER 3 ERROR : ");
				break;
			case 0x34:
				execution_get_received_message();
				if(uart0_read_data[9] == 0x34)
					DispStr1_line("MASTER 4 DATA : ");
				else
					DispStr1_line("MASTER 4 ERROR : ");
				break;
			case 0x35:
				execution_get_received_message();
				if(uart0_read_data[9] == 0x35)
					DispStr1_line("MASTER 5 DATA : ");
				else
					DispStr1_line("MASTER 5 ERROR : ");
				break;
			case 0x36:
				execution_get_received_message();
				if(uart0_read_data[9] == 0x36)
					DispStr1_line("MASTER 6 DATA : ");
				else
					DispStr1_line("MASTER 6 ERROR : ");
				break;
			case 0x37:
				execution_get_received_message();
				if(uart0_read_data[9] == 0x37)
					DispStr1_line("MASTER 7 DATA : ");
				else
					DispStr1_line("MASTER 7 ERROR : ");
				break;
			default:
				message_value = DATA_ERROR;
				DispStr1_line("DATA ERROR : ");
				uart0_length = Check_Rx_Buf0();
				for(i = 0;i<uart0_length;i++)PutChar1(GetChar0());
				break;
		}
		execution_put_received_message();
		init_uart0_read_data();
	}
}
/*
********************************************************************************
* Function For slave routine
********************************************************************************
*/
void execution_slave(void)
{
	unsigned int i;

	status_sequence = BEFORE_CONNECT;
	step_slave_bt = STEP_1;
	old_switch_value = 0;
	stream_status = ERROR;
	init_uart0_read_data();
	write_port_value(WRITE_S_CONTROL_HIGH);
	while(1)
	{
		if(status_sequence == BEFORE_CONNECT)
		{
			uart0_length = Check_Rx_Buf0();
			if(uart0_length>0)
			{
				if(lf_check_flag == 2)
				{
					uart0_length = Check_Rx_Buf0();
					for(i = 0;i<uart0_length;i++)
						uart0_read_data[i] = GetChar0();
					compare_slave_command();
				}
			}
		}
		else if(status_sequence == AFTER_CONNECT)
		{
			lf_check_flag = LF_CHECK_NON;
			execution_distribute_received_message();
			execution_stream_control();
		}
	}
}
/*
********************************************************************************
* Function For compare master command
********************************************************************************
*/
void compare_master_command(void)
{
	unsigned int i;

	switch(step_master_bt)
	{
		case STEP_1:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nBTWIN Slave mode start\r\n",&uart0_read_data[uart0_length-26],26))
			{
				step_master_bt++;
				DispStr1("[STEP_1 : START MSG RECEIVED]");
			}
			else if(!memcmp("\r\nBTWIN Master mode start\r\n",&uart0_read_data[uart0_length-27],27))
			{
				step_master_bt = STEP_3;
				DispStr1("[STEP_1 : START MSG RECEIVED]");
			}
			else
				execution_error();
			break;
		case STEP_2:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_master_bt++;
				DispStr1("[STEP_2 : OK MSG RECEIVED]");
				DispStr1("[         CHANGE ROLE]");				
				DispStr0("at+btrole=m\r");
			}
			else
				execution_error();
			break;
		case STEP_3:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_master_bt++;
				status_target = TARGET_DETECT;
				DispStr1("[STEP_3 : OK MSG RECEIVED]");
				DispStr1("[         SET OPMODE 0]");
				wait_1ms(100);
				DispStr0("at+btopmode,0\r");
			}
			else
				execution_error();
			break;
		case STEP_4:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_master_bt++;
				DispStr1("[STEP_4 : OK MSG RECEIVED]");
				DispStr1("[         RESET BLUETOOTH]");				
				DispStr0("atz\r");
			}
			else
				execution_error();
			break;
		case STEP_5:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nBTWIN Master mode start\r\n",&uart0_read_data[uart0_length-27],27))
			{
				step_master_bt++;
				DispStr1("[STEP_5 : START MSG RECEIVED]");
			}
			else
				execution_error();
			break;
		case STEP_6:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_master_bt++;
				status_target = TARGET_DETECT;
				DispStr1("[STEP_6 : OK MSG RECEIVED]");
				DispStr1_line("[         TRY TO CONNECT ");
				for(i = 0;i < 12;i++)
						PutChar1(target_data[i]);
				DispStr1("]");
				wait_1ms(100);
				DispStr0("atd");
				for(i = 0;i<12;i++)
					PutChar0(target_data[i]);
				PutChar0(0x0d);
			}
			else
				execution_error();
			break;
		case STEP_7:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nOK\r\n",&uart0_read_data[uart0_length-6],6))
			{
				step_master_bt++;
				DispStr1("[STEP_7 : OK MSG RECEIVED]");
				DispStr1("[         WAIT CONNECT MSG]");
			}
			else
				execution_error();
			break;
		case STEP_8:
			lf_check_flag = LF_CHECK_NON;
			if(!memcmp("\r\nCONNECT",&uart0_read_data[uart0_length-24],9))
			{
				step_master_bt++;
				status_sequence = AFTER_CONNECT;
		// ̽  1 ߰ Ǹ  ۽  400ms ߰(1 : 400ms, 2 : 800ms, 7 : 2800ms)
				transfer_value = ((4*(CONNECT_DEV_NUM-0x30)))*34;
				DispStr1("[STEP_8 : COMMUNICATION START]");
			}
			else if(!memcmp("\r\nERROR\r\n",&uart0_read_data[uart0_length-9],9))
			{
				step_master_bt = STEP_7;
				status_target = TARGET_DETECT;
				DispStr1("[STEP_8 : ERROR MSG RECEIVED]");
				DispStr1_line("[         TRY TO RE-CONNECT ");
				for(i = 0;i < 12;i++)
						PutChar1(target_data[i]);
				DispStr1("]");
				wait_1ms(100);
				DispStr0("atd");
				for(i = 0;i<12;i++)
					PutChar0(target_data[i]);
				PutChar0(0x0d);
			}
			else
				execution_error();
			break;
	}
	init_uart0_read_data();
}
/*
********************************************************************************
* Function For transfer master data
********************************************************************************
*/
execution_transfer_data(void)
{
	read_avr_port = read_port_value(READ_SWITCH);
	if(read_avr_port == SWITCH_1||read_avr_port == SWITCH_2||
		read_avr_port == SWITCH_3||read_avr_port == SWITCH_4)
	{
		old_switch_value = read_avr_port;
	}
	if(old_switch_value == SWITCH_1)
	{
		if(transfer_interval > transfer_value)
		{
			transfer_interval = 0;
			read_avr_port = read_port_value(READ_DIP)&0xe0;
			if(read_avr_port == DIP_2_2)
			{
				DispStr0_line("1ABCDEFGH1");
				DispStr1("SENDING DATA : 1ABCDEFGH1");
			}
			else if(read_avr_port == DIP_2_4)
			{
				DispStr0_line("2ABCDEFGH2");
				DispStr1("SENDING DATA : 2ABCDEFGH2");
			}
			else if(read_avr_port == DIP_2_3)
			{
				DispStr0_line("3ABCDEFGH3");
				DispStr1("SENDING DATA : 3ABCDEFGH3");
			}
			else if(read_avr_port == DIP_2_3_4)
			{
				DispStr0_line("4ABCDEFGH4");
				DispStr1("SENDING DATA : 4ABCDEFGH4");
			}
		}
	}	
}
/*
********************************************************************************
* Function For master routine
********************************************************************************
*/
void execution_master(void)
{
	unsigned int i;

	//Write Slave Bluetooth Device Address
	memcpy(target_data,"00025b00a5a5",12);
	//
	status_sequence = BEFORE_CONNECT;
	step_master_bt = STEP_1;
	init_uart0_read_data();
	while(1)
	{
		if(status_sequence == BEFORE_CONNECT)
		{
			uart0_length = Check_Rx_Buf0();
			if(uart0_length>0)
			{
				if(lf_check_flag == 2)
				{
					uart0_length = Check_Rx_Buf0();
					for(i = 0;i<uart0_length;i++)
						uart0_read_data[i] = GetChar0();
					compare_master_command();
				}
			}
		}
		else if(status_sequence == AFTER_CONNECT)
		{
			lf_check_flag = LF_CHECK_NON;
			execution_distribute_received_message();
			execution_transfer_data();
		}
	}
}
/*
********************************************************************************
* main Start
********************************************************************************
*/
int main(void)
{

	init_port();
	Init_UART1(BAUD_9600);
	Init_UART0(BAUD_9600);
	init_timer0();
	sei();

	read_avr_port = 0;
	lf_check_flag = LF_CHECK_NON;
	status_target = TARGET_NOT_DETECT;
	//FB755AX
	connect_count = 0x30;
	//
	wait_1ms(10);
	DispStr1("\r\n\r\nFBD755MC_gcc  START");
	while(1)
	{
		read_avr_port = read_port_value(READ_DIP)&0x30;
		if(read_avr_port == DIP_1)
			execution_slave();
		else if(read_avr_port == DIP_2)
			execution_master();
		read_avr_port = read_port_value(READ_DIP)&0x80;
		if(read_avr_port == DIP_4)
			get_bluetooth_address();
	}
}
/*
**************************************************************************************************************
* main End
**************************************************************************************************************
*/

